/*   ACUA - Access Control and User Administration.
 *   Copyright (C) 2000  Robert Davidson.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   You may not sell ACUA or any part of ACUA for profit.
 */

#ifndef HASH_H
#define HASH_H

#include "types.h"

typedef word               (*HashFn)(word size, void *key);
  
typedef struct HashTableListNode {
  void        *item;
  struct HashTableListNode  *next;
} HashTableListNode;

typedef struct HashTable {
  word                     size;
  word                     nItems;
  HashTableListNode      **table;
  HashFn                   hashFn;
  CompFn                   compFn;
  word                     keyOffset;
  Boolean                  ownsItems;
  word                     iterationList;
  HashTableListNode       *curNode;
} HashTable;

void                     hashTableInit(HashTable *hTable, word size, HashFn hashFn,
                                       CompFn compFn, word keyOffset, Boolean ownsItems);
void                     hashTableDeinit(HashTable *hTable);
word                     hashTableNitems();
void                     hashTableIterateInit(HashTable *hTable);
void                    *hashTableIterate(HashTable *hTable);
void                     hashTableIterateDeinit(HashTable *hTable);
void                     hashTableAdd(HashTable *hTable, void *item);
int                      hashTableDel(HashTable *hTable, void *key);
void                    *hashTableSearch(HashTable *hTable, void *key);
void                    *hashTableSearchNext(HashTable *hTable, void *key);

#endif
